local drawBridge = ...

drawBridge:registerAssetProcessor("models/drawBridges/drawBridgeA.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
drawBridge:registerAssetProcessor("models/drawBridges/drawBridgeB.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})
drawBridge:registerAssetProcessor("models/drawBridges/harrow.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

--------------------------------------------------------------------------

-- Register IDs for all prefab nodes referenced here
drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/sgv5dba_CenterPart", StoneGateSetPrefix .. "_DRAW_BRIDGE_A_CENTER_PREFAB")
drawBridge:registerAssetId("models/drawBridges/drawBridgeB.fbx/Prefab/drawBridge/sgv5dbb_CenterPart", StoneGateSetPrefix .. "_DRAW_BRIDGE_B_CENTER_PREFAB")
drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/RootPart", StoneGateSetPrefix .. "_DRAW_BRIDGE_CENTER_ROOT_PREFAB")
drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/StartSlopePart", StoneGateSetPrefix .. "_DRAW_BRIDGE_SLOPE_START_PREFAB")
drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/EndSlopePart", StoneGateSetPrefix .. "_DRAW_BRIDGE_SLOPE_END_PREFAB")
drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/StartPart", StoneGateSetPrefix .. "_DRAW_BRIDGE_START_PREFAB")
drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/EndPart", StoneGateSetPrefix .. "_DRAW_BRIDGE_END_PREFAB")
drawBridge:registerAssetId("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/TilingPart", StoneGateSetPrefix .. "_DRAW_BRIDGE_TILING_PREFAB")
drawBridge:registerAssetId("models/drawBridges/harrow.fbx/Prefab/drawBridge/HarrowPart", StoneGateSetPrefix .. "_DRAW_BRIDGE_HARROW_PREFAB")

--------------------------------------------------------------------------
if (foundation.getGameVersion == nil or version.cmp(foundation.getGameVersion(), "1.6") < 0) then
    drawBridge:log("Hello World: version 1.5")
    _MoverStart = "BUILDING_PART_MOVER"
    _MoverCore = "BUILDING_PART_MOVER_BRIDGE"
    _ConstructorDataStart = {
		DataType = "BUILDING_CONSTRUCTOR_BASEMENT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_DRAW_BRIDGE_START_PREFAB",
		FillerList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_TILING_PART"
		},
		BasementScale = 2
    }
    _ConstructorDataEnd =  {
		DataType = "BUILDING_CONSTRUCTOR_BASEMENT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_DRAW_BRIDGE_END_PREFAB",
		FillerList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_TILING_PART"
		},
		BasementScale = 2
    }
else
    drawBridge:log("Hello World: version 1.6")
    _MoverStart = "BUILDING_PART_MOVER_BRIDGE"
    _MoverCore = "BUILDING_PART_MOVER"
    _ConstructorDataStart =  {
		DataType = "BUILDING_CONSTRUCTOR_SLOPE",
		CoreObjectPrefab = StoneGateSetPrefix .. "_DRAW_BRIDGE_START_PREFAB",
		SlopePartList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_SLOPE_START_PART"
		},
		CoreBasementPartList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_TILING_PART"
		},
		UpperAttachType = ATTACH_NODE_TYPE.RIGHT,
		LowerAttachType = ATTACH_NODE_TYPE.LEFT
    }
    _ConstructorDataEnd = {
		DataType = "BUILDING_CONSTRUCTOR_SLOPE",
		CoreObjectPrefab = StoneGateSetPrefix .. "_DRAW_BRIDGE_END_PREFAB",
		SlopePartList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_SLOPE_END_PART"
		},
		CoreBasementPartList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_TILING_PART"
		},
		UpperAttachType = ATTACH_NODE_TYPE.LEFT,
		LowerAttachType = ATTACH_NODE_TYPE.RIGHT,
    } 
end

-- Register bridge tiling (the pillar part that extends to the ground)
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_DRAW_BRIDGE_TILING_PART",
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_DRAW_BRIDGE_TILING_PREFAB"
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 1 }
		}
	},
})

-- Register bridge start part
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_DRAW_BRIDGE_START_PART",
	Mover = { DataType = _MoverStart },
	ConstructorData = _ConstructorDataStart,
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 2 }
		},
	},
})

-- Register bridge end part
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_DRAW_BRIDGE_END_PART",
	Mover = { DataType = "BUILDING_PART_MOVER_BRIDGE" },
	ConstructorData = _ConstructorDataEnd,
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 2 }
		},
	},
})

-- Register bridge start slope part
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_DRAW_BRIDGE_SLOPE_START_PART",
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BASEMENT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_DRAW_BRIDGE_SLOPE_START_PREFAB",
		FillerList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_TILING_PART"
		},
		BasementScale = 2,
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 1 }
		},
	},
})

-- Register bridge end slope part
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_DRAW_BRIDGE_SLOPE_END_PART",
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BASEMENT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_DRAW_BRIDGE_SLOPE_END_PREFAB",
		FillerList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_TILING_PART"
		},
		BasementScale = 2,
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "STONE", Quantity = 1 }
		},
	},
})

-- Register bridge center A top part
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_DRAW_BRIDGE_A_CENTER_TOP_PART",
        BuildingZone = _DrawBridgeBuildingZone,
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_DRAW_BRIDGE_A_CENTER_PREFAB"
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "TOOL", Quantity = 2 },
			{ Resource = "PLANK", Quantity = 10 }
		},
	},
})

-- Register bridge center B top part
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_DRAW_BRIDGE_B_CENTER_TOP_PART",
        BuildingZone = _DrawBridgeBuildingZone,
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_DRAW_BRIDGE_B_CENTER_PREFAB"
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "TOOL", Quantity = 2 },
			{ Resource = "PLANK", Quantity = 10 }
		},
	},
})

-- Register the center root A part (the scalable part that places multiple center top parts)
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_DRAW_BRIDGE_A_CENTER_CORE_PART",
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER",
		CoreObjectPrefab = StoneGateSetPrefix .. "_DRAW_BRIDGE_CENTER_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_DRAW_BRIDGE_A_CENTER_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_TILING_PART"
		},
		BasementScale = 2,
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "PLANK", Quantity = 1 }
		},
	},
})

-- Register the center root B part (the scalable part that places multiple center top parts)
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_DRAW_BRIDGE_B_CENTER_CORE_PART",
	Mover = { DataType = "BUILDING_PART_MOVER" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_SCALER",
		CoreObjectPrefab = StoneGateSetPrefix .. "_DRAW_BRIDGE_CENTER_ROOT_PREFAB",
		EndPart = StoneGateSetPrefix .. "_DRAW_BRIDGE_B_CENTER_TOP_PART",
		FillerList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_TILING_PART"
		},
		BasementScale = 2,
	},
	Cost = {
		DataType = "BUILDING_PART_COST",
		RessourcesNeeded = {
			{ Resource = "PLANK", Quantity = 1 }
		},
	},
})

-- Register the core A part (the part with the bridge mover)
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_DRAW_BRIDGE_A_CORE_PART",
	Name = StoneGateSetPrefix .. "_DRAW_BRIDGE_A_CORE_NAME",
	Mover = { DataType = _MoverCore },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BRIDGE",
		StartPart = StoneGateSetPrefix .. "_DRAW_BRIDGE_START_PART",
		EndPart = StoneGateSetPrefix .. "_DRAW_BRIDGE_END_PART",
		FillerList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_A_CENTER_CORE_PART"
		},
		SlopeFillerList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_SLOPE_END_PART"
		},
		SlopeStartFillerList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_SLOPE_START_PART"
		},
		MinimumScale = 1,
                MinimumHeightAboveGround = 0
	},
	AssetBuildingFunction = StoneGateSetPrefix .. "_DRAW_BRIDGE_FUNCTION"
})
drawBridge:register({
		DataType = "BUILDING_FUNCTION_BRIDGE",
                Id = StoneGateSetPrefix .. "_DRAW_BRIDGE_FUNCTION"
})


-- Register the core B part (the part with the bridge mover)
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_DRAW_BRIDGE_B_CORE_PART",
	Name = StoneGateSetPrefix .. "_DRAW_BRIDGE_B_CORE_NAME",
	Mover = { DataType = _MoverCore },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_BRIDGE",
		StartPart = StoneGateSetPrefix .. "_DRAW_BRIDGE_START_PART",
		EndPart = StoneGateSetPrefix .. "_DRAW_BRIDGE_END_PART",
		FillerList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_B_CENTER_CORE_PART"
		},
		SlopeFillerList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_SLOPE_END_PART"
		},
		SlopeStartFillerList = {
			StoneGateSetPrefix .. "_DRAW_BRIDGE_SLOPE_START_PART"
		},
		MinimumScale = 1,
	},
	AssetBuildingFunction = StoneGateSetPrefix .. "_DRAW_BRIDGE_FUNCTION"
})

-- Register the additional harrow part
drawBridge:register({
	DataType = "BUILDING_PART",
	Id = StoneGateSetPrefix .. "_DRAW_BRIDGE_HARROW_PART",
	Name = StoneGateSetPrefix .. "_DRAW_BRIDGE_HARROW_NAME",
	Description = StoneGateSetPrefix .. "_DRAW_BRIDGE_HARROW_DESC",
	Category = "EXTENSION",
	Mover = { DataType = "BUILDING_PART_MOVER_INSTANCE" },
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = StoneGateSetPrefix .. "_DRAW_BRIDGE_HARROW_PREFAB",
                IsPreviewGroundVisible = false
	},
	Cost = {
		UpkeepCost = {
			{ Resource = "GOLD", Quantity = 0 }
		},
		RessourcesNeeded = { -- List of RESOURCE_QUANTITY_PAIR
			{ Resource = "TOOL", Quantity = 1 }
		}
	}
    })


--------------------------------------------------------------------------

-- Override building part components properties (generated by the building asset processor) to add building zones

drawBridge:registerPrefabComponent("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/sgv5dba_CenterPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 2 }
})
drawBridge:registerPrefabComponent("models/drawBridges/drawBridgeB.fbx/Prefab/drawBridge/sgv5dbb_CenterPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 2, 2 }
})
drawBridge:registerPrefabComponent("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/StartPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 0, 0 }
})
drawBridge:registerPrefabComponent("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/EndPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 0, 0 }
})
drawBridge:registerPrefabComponent("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/StartSlopePart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 0, 0 }
})
drawBridge:registerPrefabComponent("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/EndSlopePart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = true,
	BuildingZone = { 0, 0 }
})
drawBridge:registerPrefabComponent("models/drawBridges/harrow.fbx/Prefab/drawBridge/HarrowPart", {
	DataType = "COMP_BUILDING_PART",
	HasBuildingZone = false,
	BuildingZone = { 0, 0 }
})

--------------------------------------------------------------------------

-- Configure all BRIDGE colliders

drawBridge:configurePrefabFlagList("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/sgv5dba_CenterPart", { "BRIDGE" })
drawBridge:configurePrefabFlagList("models/drawBridges/drawBridgeB.fbx/Prefab/drawBridge/sgv5dbb_CenterPart", { "BRIDGE" })
drawBridge:configurePrefabFlagList("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/StartPart", { "BRIDGE" })
drawBridge:configurePrefabFlagList("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/EndPart", { "BRIDGE" })
drawBridge:configurePrefabFlagList("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/StartSlopePart", { "BRIDGE" })
drawBridge:configurePrefabFlagList("models/drawBridges/drawBridgeA.fbx/Prefab/drawBridge/EndSlopePart", { "BRIDGE" })
